package com.ym.mec.biz.service.impl;


import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetCategoryWrapper;
import com.dayaedu.cbs.openfeign.wrapper.music.CbsMusicSheetWrapper;
import com.ym.mec.auth.api.dto.MusicScoreQueryInfo;
import com.ym.mec.biz.dal.dao.SysMusicScoreAccompanimentDao;
import com.ym.mec.biz.dal.entity.*;
import com.ym.mec.biz.dal.page.SysExamSongQueryInfo;
import com.ym.mec.biz.service.InstrumentService;
import com.ym.mec.biz.service.SysMusicScoreAccompanimentService;
import com.ym.mec.biz.service.SysMusicScoreCategoriesService;
import com.ym.mec.biz.service.SysMusicScoreService;
import com.ym.mec.common.dal.BaseDAO;
import com.ym.mec.common.exception.BizException;
import com.ym.mec.common.page.PageInfo;
import com.ym.mec.common.page.PageUtil;
import com.ym.mec.common.service.impl.BaseServiceImpl;
import com.ym.mec.util.collection.MapUtil;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@Service
public class SysMusicScoreAccompanimentServiceImpl extends BaseServiceImpl<String, SysMusicScoreAccompaniment> implements SysMusicScoreAccompanimentService {
	
	@Resource
	private SysMusicScoreAccompanimentDao sysMusicScoreAccompanimentDao;
	@Resource
	private SysMusicScoreCategoriesService sysMusicScoreCategoriesService;
	@Resource
	private SysMusicScoreService sysMusicScoreService;
	@Resource
	private InstrumentService instrumentService;
	@Override
	public BaseDAO<String, SysMusicScoreAccompaniment> getDAO() {
		return sysMusicScoreAccompanimentDao;
	}


	@Override
	public List<SysMusicScoreAccompaniment> queryAll(SysExamSongQueryInfo queryInfo) {
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
		return this.initAccompaniment(sysMusicScoreAccompanimentDao.queryPage(params));
	}

	public List<SysMusicScoreAccompaniment> initAccompaniment(List<SysMusicScoreAccompaniment> accompaniments){
		List<SysMusicScoreAccompaniment> result = new ArrayList<>();
		if(CollectionUtils.isEmpty(accompaniments)){
			return result;
		}
		long count = accompaniments.stream().filter(e -> e.getCbsMusicSheetId() != null).count();
		if(count == 0){
			return result;
		}
		List<String> cbsMusicSheetIds = accompaniments.stream().map(SysMusicScoreAccompaniment::getCbsMusicSheetId)
				.filter(StringUtils::isNotEmpty).collect(Collectors.toList());
		List<CbsMusicSheetWrapper.MusicSheetApplication> applications = sysMusicScoreService.queryCbsMusicSheetApplication(cbsMusicSheetIds);
		if (CollectionUtils.isEmpty(applications)) {
			throw new BizException("获取曲目信息失败");
		}
		Map<String, CbsMusicSheetWrapper.MusicSheetApplication> applicationMap = applications.stream().collect(Collectors.toMap(e->e.getId().toString(), Function.identity()));
		for (SysMusicScoreAccompaniment accompaniment : accompaniments) {
			if(accompaniment.getCbsMusicSheetId() == null){
				result.add(accompaniment);
				continue;
			}
			CbsMusicSheetWrapper.MusicSheetApplication musicSheetApplication = applicationMap.get(accompaniment.getCbsMusicSheetId());
			if (musicSheetApplication == null) {
				throw new BizException("曲目信息不存在，曲目ID：" + accompaniment.getCbsMusicSheetId());
			}
			if(CollectionUtils.isEmpty(musicSheetApplication.getMusicSheetSoundList())){
				throw new BizException("曲目原音信息不存在，曲目ID：" + accompaniment.getCbsMusicSheetId());
			}
			List<SysMusicScoreAccompaniment> childAccompaniments = new ArrayList<>();
			for (int i = 0; i < musicSheetApplication.getMusicSheetSoundList().size(); i++) {
				SysMusicScoreAccompaniment record = new SysMusicScoreAccompaniment();
				record.setIsConvertibleScore(musicSheetApplication.getIsConvertibleScore());
				if(musicSheetApplication.getScoreType() != null){
					record.setScoreType(musicSheetApplication.getScoreType().getCode());
				}
				record.setId(accompaniment.getExamSongId().toString());
				record.setType(accompaniment.getType());
				record.setRankIds("FREE".equals(musicSheetApplication.getPaymentType()) ? null : "1");
				record.setClientType(accompaniment.getClientType());
				record.setExamSongId(accompaniment.getExamSongId());
				record.setCbsMusicSheetId(musicSheetApplication.getId().toString());
				record.setPlayMode(SysMusicScore.PlayMode.valueOf(musicSheetApplication.getPlayMode().getCode()));
				record.setExtConfigJson(musicSheetApplication.getExtConfigJson());
//				record.setSubjectId(accompaniment.getSubjectId());
				record.setExtStyleConfigJson(musicSheetApplication.getExtStyleConfigJson());
				if(musicSheetApplication.getMusicSheetCategoryId() != null){
					record.setCategoriesId(musicSheetApplication.getMusicSheetCategoryId().intValue());
				}
				record.setMusicSheetType(musicSheetApplication.getMusicSheetType().getCode());
				record.setParentCategoriesId(accompaniment.getParentCategoriesId());
				record.setExamSongName(musicSheetApplication.getName());
				record.setCategoriesName(musicSheetApplication.getMusicSheetCategoryName());
				record.setSpeed(musicSheetApplication.getPlaySpeed());
				CbsMusicSheetWrapper.MusicSheetSound sound = musicSheetApplication.getMusicSheetSoundList().get(i);
				record.setMetronomeMp3Url(sound.getAudioFileUrl());
				record.setTrack(sound.getTrack());
				record.setSubjectName(sound.getTrack());
				record.setXmlUrl(musicSheetApplication.getXmlFileUrl());
				record.setMidiUrl(musicSheetApplication.getMidiFileUrl());
				record.setEnableEvaluation(musicSheetApplication.getIsEvaluated());
				record.setIsOpenMetronome(musicSheetApplication.getIsUseSystemBeat());
				List<CbsMusicSheetWrapper.MusicSheetAccompaniment> accompanimentList = musicSheetApplication.getMusicSheetAccompanimentList();
				if (CollectionUtils.isNotEmpty(accompanimentList)) {
					record.setMetronomeUrl(accompanimentList.get(0).getAudioFileUrl());
					record.setUrl(accompanimentList.get(0).getAudioFileUrl());
				}
				record.setMetronomeMp3Url(sound.getAudioFileUrl());
				record.setMp3Url(sound.getAudioFileUrl());
				record.setIsShowFingering(musicSheetApplication.getIsShowFingering());
				childAccompaniments.add(record);
			}
			result.addAll(childAccompaniments);
		}
		return result;
	}

	public List<SysMusicScoreAccompaniment> initAccompaniments(List<CbsMusicSheetWrapper.MusicSheetAccApplication> applications){
		List<SysMusicScoreAccompaniment> result = new ArrayList<>();
		if(CollectionUtils.isEmpty(applications)){
			return result;
		}
		List<Long> musicSheetCategoryIds = applications.stream().map(e -> e.getMusicSheetCategoryId())
				.filter(Objects::isNull).distinct().collect(Collectors.toList());
		CbsMusicSheetCategoryWrapper.MusicSheetCategoryQuery categoryQuery = new CbsMusicSheetCategoryWrapper.MusicSheetCategoryQuery();
		categoryQuery.setIds(musicSheetCategoryIds);
		List<CbsMusicSheetCategoryWrapper.MusicSheetCategory> musicSheetCategories = sysMusicScoreCategoriesService.getDao().categoryList(categoryQuery);
		Map<Long, CbsMusicSheetCategoryWrapper.MusicSheetCategory> categoryMap = musicSheetCategories.stream().
				collect(Collectors.toMap(CbsMusicSheetCategoryWrapper.MusicSheetCategory::getId, Function.identity()));
		for (CbsMusicSheetWrapper.MusicSheetAccApplication musicSheetApplication : applications) {
			SysMusicScoreAccompaniment record = new SysMusicScoreAccompaniment();
			record.setIsConvertibleScore(musicSheetApplication.getIsConvertibleScore());
			if(musicSheetApplication.getScoreType() != null){
				record.setScoreType(musicSheetApplication.getScoreType().getCode());
			}
			record.setId(musicSheetApplication.getMusicSheetSoundId().toString());
			record.setRankIds("FREE".equals(musicSheetApplication.getPaymentType()) ? null : "1");
//			record.setExamSongId(accompaniment.getExamSongId());
			record.setCbsMusicSheetId(musicSheetApplication.getId().toString());
			record.setPlayMode(SysMusicScore.PlayMode.valueOf(musicSheetApplication.getPlayMode().getCode()));
			record.setExtConfigJson(musicSheetApplication.getExtConfigJson());
//				record.setSubjectId(accompaniment.getSubjectId());
			record.setExtStyleConfigJson(musicSheetApplication.getExtStyleConfigJson());
			if(musicSheetApplication.getMusicSheetCategoryId() != null){
				record.setCategoriesId(musicSheetApplication.getMusicSheetCategoryId().intValue());
				CbsMusicSheetCategoryWrapper.MusicSheetCategory musicSheetCategory = categoryMap.get(record.getCategoriesId().longValue());
				if(musicSheetCategory != null){
					record.setParentCategoriesId(musicSheetCategory.getParentId().intValue());
				}
			}
			record.setMusicSheetType(musicSheetApplication.getMusicSheetType().getCode());
			record.setExamSongName(musicSheetApplication.getName());
			record.setCategoriesName(musicSheetApplication.getMusicSheetCategoryName());
			record.setSpeed(musicSheetApplication.getPlaySpeed());
			//原音
			record.setMetronomeMp3Url(musicSheetApplication.getMp3Url());
			record.setMp3Url(musicSheetApplication.getMp3Url());
			record.setTrack(musicSheetApplication.getTrack());
			record.setSubjectName(musicSheetApplication.getTrack());
			record.setXmlUrl(musicSheetApplication.getXmlFileUrl());
			record.setMidiUrl(musicSheetApplication.getMidiFileUrl());
			record.setEnableEvaluation(musicSheetApplication.getIsEvaluated());
			record.setIsOpenMetronome(musicSheetApplication.getIsUseSystemBeat());
			//伴奏
			record.setMetronomeUrl(musicSheetApplication.getUrl());
			record.setUrl(musicSheetApplication.getUrl());
			record.setIsShowFingering(musicSheetApplication.getIsShowFingering());
			result.add(record);
		}
		return result;
	}

	@Override
	public SysMusicScoreAccompaniment initSysMusicScoreAccompaniment(CbsMusicSheetWrapper.MusicSheetAccApplication musicSheetApplication){
		SysMusicScoreAccompaniment record = new SysMusicScoreAccompaniment();
		record.setIsConvertibleScore(musicSheetApplication.getIsConvertibleScore());
		if(musicSheetApplication.getScoreType() != null){
			record.setScoreType(musicSheetApplication.getScoreType().getCode());
		}
//		record.setType(accompaniment.getType());
		record.setRankIds("FREE".equals(musicSheetApplication.getPaymentType()) ? null : "1");
//		record.setClientType(accompaniment.getClientType());
//		record.setExamSongId(accompaniment.getExamSongId());
		record.setCbsMusicSheetId(musicSheetApplication.getId().toString());
		record.setPlayMode(SysMusicScore.PlayMode.valueOf(musicSheetApplication.getPlayMode().getCode()));
		record.setExtConfigJson(musicSheetApplication.getExtConfigJson());
//				record.setSubjectId(accompaniment.getSubjectId());
		record.setExtStyleConfigJson(musicSheetApplication.getExtStyleConfigJson());
		if(musicSheetApplication.getMusicSheetCategoryId() != null){
			record.setCategoriesId(musicSheetApplication.getMusicSheetCategoryId().intValue());
		}
		record.setMusicSheetType(musicSheetApplication.getMusicSheetType().getCode());
//		record.setParentCategoriesId(accompaniment.getParentCategoriesId());
		record.setExamSongName(musicSheetApplication.getName());
		record.setCategoriesName(musicSheetApplication.getMusicSheetCategoryName());
		record.setSpeed(musicSheetApplication.getPlaySpeed());
		record.setTrack(musicSheetApplication.getTrack());
		record.setSubjectName(musicSheetApplication.getTrack());
		record.setXmlUrl(musicSheetApplication.getXmlFileUrl());
		record.setMidiUrl(musicSheetApplication.getMidiFileUrl());
		record.setEnableEvaluation(musicSheetApplication.getIsEvaluated());
		record.setIsOpenMetronome(musicSheetApplication.getIsUseSystemBeat());
		record.setMetronomeUrl(musicSheetApplication.getUrl());
		record.setUrl(musicSheetApplication.getUrl());
		record.setMetronomeMp3Url(musicSheetApplication.getMp3Url());
		record.setMp3Url(musicSheetApplication.getMp3Url());
		record.setIsShowFingering(musicSheetApplication.getIsShowFingering());
		return record;
	}

	@Override
	public List<SysMusicScoreAccompaniment> queryAccPage(SysExamSongQueryInfo queryInfo) {
		if(queryInfo.getCategoriesId() != null){
			List<Integer> categoriesIdList = new ArrayList<>();
			MusicScoreQueryInfo musicScoreQueryInfo = new MusicScoreQueryInfo();
			Integer categoriesId = null;
			if(StringUtils.isNotEmpty(queryInfo.getCategoriesId())){
				categoriesId = Integer.parseInt(queryInfo.getCategoriesId());
			}
			musicScoreQueryInfo.setParentId(categoriesId);
			musicScoreQueryInfo.setEnable(queryInfo.getEnable());
			musicScoreQueryInfo.setOrganId(queryInfo.getOrganId());
			List<SysMusicScoreCategories> scoreCategories = sysMusicScoreCategoriesService.queryTree(musicScoreQueryInfo);
			if(CollectionUtils.isNotEmpty(scoreCategories)){
				categoriesIdList = scoreCategories.stream().map(SysMusicScoreCategories::getId).collect(Collectors.toList());
			}else {
				categoriesIdList.add(categoriesId);
			}
			queryInfo.setCategoriesIdList(categoriesIdList);
		}
		Map<String, Object> params = new HashMap<>();
		MapUtil.populateMap(params, queryInfo);
        List<SysMusicScoreAccompaniment> sysMusicScoreAccompaniments = sysMusicScoreAccompanimentDao.queryAccPage(params);
        for (SysMusicScoreAccompaniment sysMusicScoreAccompaniment : sysMusicScoreAccompaniments) {
            if (sysMusicScoreAccompaniment.getIsOpenMetronome() ==null || sysMusicScoreAccompaniment.getIsOpenMetronome()) {
                if (StringUtils.isNotBlank(sysMusicScoreAccompaniment.getUrl())) {
                    sysMusicScoreAccompaniment.setMetronomeUrl(sysMusicScoreAccompaniment.getUrl());
                }
            }
        }
        return this.initAccompaniment(sysMusicScoreAccompaniments);
    }

	@Override
	public List<Subject> querySubjectIds(Integer categoriesId) {
		List<Integer> categoriesIdList = new ArrayList<>();
		if(categoriesId != null){
			MusicScoreQueryInfo musicScoreQueryInfo = new MusicScoreQueryInfo();
			musicScoreQueryInfo.setParentId(categoriesId);
			List<SysMusicScoreCategories> scoreCategories = sysMusicScoreCategoriesService.queryTree(musicScoreQueryInfo);
			if(CollectionUtils.isNotEmpty(scoreCategories)){
                List<SysMusicScoreCategories> allScoreCategories = new ArrayList<>(scoreCategories);
				for (SysMusicScoreCategories scoreCategory : scoreCategories) {
					if(scoreCategory.getSysMusicScoreCategoriesList() != null){
						allScoreCategories.addAll(scoreCategory.getSysMusicScoreCategoriesList());
					}
				}
				categoriesIdList = allScoreCategories.stream().map(SysMusicScoreCategories::getId).collect(Collectors.toList());
			}else {
				categoriesIdList.add(categoriesId);
			}
		}
		return sysMusicScoreAccompanimentDao.querySubjectIds(categoriesIdList);
	}

    @Override
    public PageInfo<SysMusicScoreAccompaniment> queryScorePage(SysExamSongQueryInfo queryInfo) {
		if(queryInfo.getCategoriesIds() != null){
			List<Integer> categoriesIdList = new ArrayList<>();
			String[] categoriesIds = queryInfo.getCategoriesIds().split(",");

			for(String fc : categoriesIds){
				SysMusicScoreCategories sysMusicScoreCategories = sysMusicScoreCategoriesService.get(Integer.parseInt(fc));
				
				if(sysMusicScoreCategories == null){
					throw new BizException("分类找不到");
				}
				categoriesIdList.add(Integer.parseInt(fc));
				sysMusicScoreCategories = sysMusicScoreCategoriesService.getChildTree(sysMusicScoreCategories);
				getAllCategoryIdList(categoriesIdList, sysMusicScoreCategories.getSysMusicScoreCategoriesList());
			}
			queryInfo.setCategoriesIds(categoriesIdList.stream().map(String :: valueOf).collect(Collectors.joining(",")));
		}
		if(StringUtils.isNotBlank(queryInfo.getFilterCategoriesIds())){
			String[] filterCategoriesIds = queryInfo.getFilterCategoriesIds().split(",");

			List<Integer> filterCategoriesIdList = new ArrayList<>();
			
			for(String fc : filterCategoriesIds){
				SysMusicScoreCategories sysMusicScoreCategories = sysMusicScoreCategoriesService.get(Integer.parseInt(fc));
				
				if(sysMusicScoreCategories == null){
					throw new BizException("分类找不到");
				}
				filterCategoriesIdList.add(Integer.parseInt(fc));
				sysMusicScoreCategories = sysMusicScoreCategoriesService.getChildTree(sysMusicScoreCategories);
				getAllCategoryIdList(filterCategoriesIdList, sysMusicScoreCategories.getSysMusicScoreCategoriesList());
			}
			queryInfo.setFilterCategoriesIds(filterCategoriesIdList.stream().map(String :: valueOf).collect(Collectors.joining(",")));
		}
		//如果声部不为空，关联声部挂的乐器
		List<Instrument> list = null;
		if(queryInfo.getSubjectId() != null){
			list = instrumentService.lambdaQuery().eq(Instrument::getSubjectId, queryInfo.getSubjectId()).list();
		}else {
			//获取所有有关联的乐器
			list = instrumentService.lambdaQuery().list();
		}
		if(CollectionUtils.isEmpty(list)){
			return new PageInfo<>();
		}
		CbsMusicSheetWrapper.MusicSheetApplicationQuery applicationQuery = sysMusicScoreService.getMusicSheetApplicationQuery();
		applicationQuery.setCategoriesIds(queryInfo.getCategoriesIds());
		applicationQuery.setFilterCategoriesIds(queryInfo.getFilterCategoriesIds());
		applicationQuery.setMusicalInstrumentIds(list.stream().map(e->e.getId().longValue()).collect(Collectors.toList()));
		applicationQuery.setPage(queryInfo.getPage());
		applicationQuery.setRows(queryInfo.getRows());
		applicationQuery.setKeyword(queryInfo.getSearch());
		PageInfo<CbsMusicSheetWrapper.MusicSheetAccApplication> musicSheetAccApplicationPageInfo = sysMusicScoreService.queryCbsMusicSheetSoundApplication(applicationQuery);
		List<SysMusicScoreAccompaniment> accompaniments = this.initAccompaniments(musicSheetAccApplicationPageInfo.getRows());
		return PageUtil.transPageInfo(musicSheetAccApplicationPageInfo,accompaniments);
	}
    
    private void getAllCategoryIdList(List<Integer> categoryIdList, List<SysMusicScoreCategories> sysMusicScoreCategoriesList){
    	
    	if(CollectionUtils.isNotEmpty(sysMusicScoreCategoriesList)){
    		for(SysMusicScoreCategories sysMusicScoreCategories : sysMusicScoreCategoriesList){
    			categoryIdList.add(sysMusicScoreCategories.getId());
    			getAllCategoryIdList(categoryIdList, sysMusicScoreCategories.getSysMusicScoreCategoriesList());
    		}
    	}
    	
    }
}